import 'package:intl/intl.dart';

final _fullDatetimeFormat = DateFormat('yyyy-MM-dd HH:mm:ss.SSS');
final _shortDatetimeFormat = DateFormat('yyyy-MM-dd HH:mm:ss');
final _dateFormat = DateFormat('yyyy-MM-dd');

/// DateTime类型扩展
extension FDateTimeExtensions on DateTime {
  /// 转换为指定格式的字符串
  ///
  /// [formatRule]指定格式，为空时，默认使用[yyyy-MM-dd HH:mm:ss]
  String format([String? formatRule]) {
    if (formatRule == null) return formatDatetime();
    return DateFormat(formatRule).format(this);
  }

  /// 转换为[yyyy-MM-dd HH:mm:ss]格式的字符串
  String formatFullDatetime() {
    this.toUtc();
    return _fullDatetimeFormat.format(this);
  }

  /// 转换为[yyyy-MM-dd HH:mm:ss]格式的字符串
  String formatDatetime() {
    return _shortDatetimeFormat.format(this);
  }

  /// 转换为[yyyy-MM-dd]格式的字符串
  String formatDate() {
    return _dateFormat.format(this);
  }
}
