import 'dart:async';

import 'dart:ui';

extension FunctionExt on Function {
  VoidCallback throttle({int? timeout}) {
    return FunctionProxy(this, timeout: timeout).throttle;
  }

  VoidCallback debounce({int? timeout}) {
    return FunctionProxy(this, timeout: timeout).debounce;
  }
}

class FunctionProxy {
  static final Map<String, bool> _funcThrottle = {};
  static final Map<String, Timer> _funcDebounce = {};

  /// Nonanonymous function
  final Function? target;

  /// Timeout ms value of waiting once, default 500ms
  final int? timeout;

  FunctionProxy(this.target, {this.timeout});

  void throttle() async {
    String key = target.hashCode.toString();
    bool enable = _funcThrottle[key] ?? true;
    if (enable) {
      _funcThrottle[key] = false;
      if (timeout != null && timeout! > 0) {
        Timer(Duration(milliseconds: timeout!), () {
          _funcThrottle.remove(key);
        });
      }
      try {
        await target?.call();
      } catch (e) {
        rethrow;
      } finally {
        _funcThrottle.remove(key);
      }
    }
  }

  void debounce() {
    String key = target.hashCode.toString();
    Timer? timer = _funcDebounce[key];
    timer?.cancel();
    timer = Timer(Duration(milliseconds: timeout ?? 500), () {
      Timer? t = _funcDebounce.remove(key);
      t?.cancel();
      target?.call();
    });
    _funcDebounce[key] = timer;
  }
}
