import 'dart:ui';
import 'dart:math' as Math;

import 'package:flutter/material.dart';

class FColorHelper {
  /// mix 2 colors with target percent
  ///
  /// [percent] 0-100
  static Color mixColor(Color colorA, Color colorB, int percent) {
    double p = Math.min(Math.max(0, percent.toDouble()), 100.0) / 100.0;
    double w = p * 2 - 1;
    int a = colorA.alpha - colorB.alpha;
    double w1 = ((w * a == -1 ? w : (w + a) / (1 + w * a)) + 1) / 2.0;
    double w2 = 1 - w1;

    var mixedAlpha = colorA.alpha * p + colorB.alpha * (1 - p);
    var mixedRed = w1 * colorA.red + w2 * colorB.red;
    var mixedGreen = w1 * colorA.green + w2 * colorB.green;
    var mixedBlue = w1 * colorA.blue + w2 * colorB.blue;

    Color mixedColor = Color.fromARGB(
      mixedAlpha.toInt(),
      mixedRed.toInt(),
      mixedGreen.toInt(),
      mixedBlue.toInt(),
    );
    return mixedColor;
  }

  /// get light color base target color
  ///
  /// [weight] 1-9, more bigger, more lighter
  static Color getLightColor(Color color, int weight) =>
      mixColor(const Color(0xffffffff), color, 10 * weight);

  /// Hex( #FF00CC )转Color
  ///
  /// [hexString] hex字符串
  ///
  /// [hasTag] 是否包含'#'标记
  static Color? hex2Color(String? hexString, {bool hasTag = true}) {
    if (hexString == null || hexString.isEmpty) return null;
    String hex = hasTag ? hexString.substring(1, 7) : hexString;
    return Color(int.parse(hex, radix: 16) + 0xFF000000);
  }

  /// 创建MaterialColor
  static MaterialColor createMaterialColor(Color color) {
    List strengths = <double>[.05];
    Map<int, Color> swatch = {};
    final int r = color.red, g = color.green, b = color.blue;

    for (int i = 1; i < 10; i++) {
      strengths.add(0.1 * i);
    }
    strengths.forEach((strength) {
      final double ds = 0.5 - strength;
      swatch[(strength * 1000).round()] = Color.fromRGBO(
        r + ((ds < 0 ? r : (255 - r)) * ds).round(),
        g + ((ds < 0 ? g : (255 - g)) * ds).round(),
        b + ((ds < 0 ? b : (255 - b)) * ds).round(),
        1,
      );
    });
    return MaterialColor(color.value, swatch);
  }
}
