import 'dart:convert';
import 'dart:typed_data';

import 'package:crypto/crypto.dart';
import 'package:encrypt/encrypt.dart';
import 'package:fis_common/helpers/string_util.dart';
import 'package:fis_common/logger/logger.dart';

/// 加密帮助类
class FEncryptHelper {
  static const _DEFAULT_AES_KEY =
      "a8878cd2ac86001751a00cd22f949be0"; // vinno123 -> md5
  // ignore: non_constant_identifier_names
  static final Uint8List _AES_IV_BYTES =
      Uint8List.fromList([0x12, 0x34, 0x56, 0x78, 0x90, 0xAB, 0xCD, 0xEF]);

  /// AES加密
  ///
  /// [text] 明文
  ///
  /// [encryptKey] 加密秘钥
  static String encrypAES(String text, {String? encryptKey}) {
    final key = Key.fromUtf8(encryptKey ?? _DEFAULT_AES_KEY);
    final iv = IV(_AES_IV_BYTES);
    final encrypter = Encrypter(AES(key));
    final encrypted = encrypter.encrypt(text, iv: iv);
    return encrypted.base64;
  }

  /// AES解密
  ///
  /// [text] 密文
  ///
  /// [decryptKey] 解密秘钥
  static String decrypAES(String text, {String? decryptKey}) {
    final key = Key.fromUtf8(decryptKey ?? _DEFAULT_AES_KEY);
    final iv = IV(_AES_IV_BYTES);
    final decrypter = Encrypter(AES(key));
    final decrypted = decrypter.decrypt64(text, iv: iv);
    return decrypted;
  }

  ///Base64加密
  /// [input] 明文
  static String encodeBase64(String input) {
    try {
      var result = "";
      for (int i = 0; i < input.length; i++) {
        result += _encodeOneBase64(input[i]);
      }
      return result;
    } catch (e) {
      print(e);
      logger.e('encodeBase64 ex:', e);
      return input;
    }
  }

  ///base64解密
  /// [input] 密文
  static String decryptBase64(String input) {
    try {
      if (input.isEmpty) {
        return input;
      }
      if ((input.length % 4) != 0) {
        return input;
      }
      final base64Code =
          "0123456789+/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ=";
      int page = (input.length / 4).floor();
      var outMessage = <int>[];
      var outStr = '';
      for (int i = 0; i < page; i++) {
        var instr = Uint8List(4);
        instr[0] = base64Code.indexOf(input[i * 4]);
        instr[1] = base64Code.indexOf(input[i * 4 + 1]);
        instr[2] = base64Code.indexOf(input[i * 4 + 2]);
        instr[3] = base64Code.indexOf(input[i * 4 + 3]);
        var outstr = Uint8List(3);
        outstr[0] = ((instr[0] << 2) ^ ((instr[1] & 0x30) >> 4));
        if (instr[2] != 64) {
          outstr[1] = ((instr[1] << 4) ^ ((instr[2] & 0x3c) >> 2));
        } else {
          outstr[2] = 0;
        }
        if (instr[3] != 64) {
          outstr[2] = ((instr[2] << 6) ^ instr[3]);
        } else {
          outstr[2] = 0;
        }
        outMessage.add(outstr[0]);
        if (outstr[1] != 0) {
          outMessage.add(outstr[1]);
        }
        if (outstr[2] != 0) {
          outMessage.add(outstr[2]);
        }
        outStr = utf8.decode(outMessage);
      }

      return outStr;
    } catch (e) {
      print(e);
      logger.e('FEncryptHelper decryptBase64 ex:', e);
      return input;
    }
  }

  ///MD5加密（单向加密）
  static String encodeMd5(String data) {
    Uint8List content = new Utf8Encoder().convert('V1N' + data + 'N0');
    Digest digest = md5.convert(content);
    return digest.toString();
  }

  ///部分加密
  static String encryptionPartial(String outStr) {
    final length = outStr.length;
    if (StringUtil.isChinaPhoneLegal(outStr)) {
      outStr = outStr.replaceRange(4, 7, "****");
    } else if (StringUtil.isChinese(outStr)) {
      if (length == 2) {
        outStr = outStr.replaceRange(1, null, "*");
      } else if (length == 3) {
        outStr = outStr.replaceRange(1, length - 1, "*");
      } else {
        var replacement = "";
        for (var i = 0; i < length - 2; i++) {
          replacement += "*";
        }
        outStr = outStr.replaceRange(1, length - 1, replacement);
      }
    } else if (outStr.contains('.')) {
      final strList = outStr.split('.');
      var replacement = "";
      for (var i = 0; i < strList[0].length; i++) {
        replacement += "*";
      }
      outStr = replacement + "." + strList[1];
    } else if (outStr.contains(' ')) {
      final strList = outStr.split(' ');
      var replacement = "";
      for (var i = 0; i < strList[0].length; i++) {
        replacement += "*";
      }
      outStr = replacement + " " + strList[1];
    } else if (length >= 2) {
      if (length == 2) {
        outStr = outStr.replaceRange(1, null, "*");
      } else if (length == 18) {
        outStr = outStr.replaceRange(6, 14, "********");
      } else {
        var replacement = "";
        var start = (length / 3).truncate();
        var end = ((length / 3) * 2).ceil();
        var range = end - start;
        for (var i = 0; i < range; i++) {
          replacement += "*";
        }
        outStr = outStr.replaceRange(start, end, replacement);
      }
    }
    return outStr;
  }

  static String _encodeOneBase64(String input) {
    List<int> list = utf8.encode(input);
    List<int> tempList = [];
    tempList.addAll(list);
    int use = 0;
    int messageLen = tempList.length;
    int page = (messageLen / 3).floor();

    use = messageLen % 3;
    if (use > 0) {
      for (int i = 0; i < 3 - use; i++) {
        tempList.add(0);
      }
      page++;
    }
    Uint8List byteMessage = Uint8List.fromList(tempList);
    String outmessage = '';
    for (int i = 0; i < page; i++) {
      Uint8List instr = Uint8List(3);
      instr[0] = byteMessage[i * 3];
      instr[1] = byteMessage[i * 3 + 1];
      instr[2] = byteMessage[i * 3 + 2];
      var outstr = <int>[];
      outstr.add(instr[0] >> 2);
      outstr.add(((instr[0] & 0x03) << 4) ^ (instr[1] >> 4));
      if (instr[1] != 0) {
        outstr.add(((instr[1] & 0x0f) << 2) ^ (instr[2] >> 6));
      } else {
        outstr.add(64);
      }
      if (instr[2] != 0) {
        outstr.add((instr[2] & 0x3f));
      } else {
        outstr.add(64);
      }
      outmessage += _utf8Encode(outstr);
    }
    return outmessage;
  }

  ///编码
  static String _utf8Encode(List<int> list) {
    final base64Code = [
      '0',
      '1',
      '2',
      '3',
      '4',
      '5',
      '6',
      '7',
      '8',
      '9',
      '+',
      '/',
      'a',
      'b',
      'c',
      'd',
      'e',
      'f',
      'g',
      'h',
      'i',
      'j',
      'k',
      'l',
      'm',
      'n',
      'o',
      'p',
      'q',
      'r',
      's',
      't',
      'u',
      'v',
      'w',
      'x',
      'y',
      'z',
      'A',
      'B',
      'C',
      'D',
      'E',
      'F',
      'G',
      'H',
      'I',
      'J',
      'K',
      'L',
      'M',
      'N',
      'O',
      'P',
      'Q',
      'R',
      'S',
      'T',
      'U',
      'V',
      'W',
      'X',
      'Y',
      'Z',
      '='
    ];
    var utftext = "";
    for (var n = 0; n < list.length; n++) {
      int c = list[n];
      utftext += base64Code[c];
    }
    return utftext;
  }
}
