import 'dart:io';
import 'dart:typed_data';

import '../logger/logger.dart';
import 'package:http/http.dart' as http;

class FHttpHelper {
  /// 以Get方式下载字节数组
  ///
  /// [url] 下载地址
  static Future<Uint8List?> downloadBytes(String url,
      {Map<String, String>? headers}) async {
    try {
      var response = await http.get(Uri.parse(url), headers: headers);
      if (response.statusCode == HttpStatus.ok) {
        return response.bodyBytes;
      }
    } catch (e) {
      logger.e("FHttpHelper 下载Byte[]异常", e);
    }
    return null;
  }

  /// 以Get方式下载字符串
  ///
  /// [url] 下载地址
  static Future<String?> downloadString(String url) async {
    try {
      var response = await http.get(Uri.parse(url), headers: {
        // "Access-Control-Allow-Origin": "*",
        // "Access-Control-Allow-Credentials": "true",
      });
      if (response.statusCode == HttpStatus.ok) {
        return response.body;
      }
    } catch (e) {
      logger.e("FHttpHelper 下载字符串异常", e);
    }
    return null;
  }
}
