import 'package:dio/dio.dart' as dio;

import 'options.dart';
import 'response.dart';
import 'entry_stub.dart'
    if (dart.library.html) 'entry/http_for_web.dart'
    if (dart.library.io) 'entry/http_for_native.dart';

export 'package:dio/src/multipart_file.dart';
export 'package:dio/src/form_data.dart';

/// Fis Http Client 封装
abstract class FHttp {
  factory FHttp([FHttpBaseOptions? options]) => createFHttp(options);

  /// 基础配置
  late FHttpBaseOptions options;

  /// 基于Dio封装，不用管，勿动
  late final dio.HttpClientAdapter httpClientAdapter;

  void close({bool force = false});

  /// GET 请求
  ///
  /// [path] 请求地址
  ///
  /// [queryParameters] 请求参数
  ///
  /// [options] 单次配置，仅本次请求有效
  ///
  /// [cancelToken] 取消标识
  Future<FHttpResponse<T>> get<T>(
    String path, {
    Map<String, dynamic>? queryParameters,
    FHttpScopedOptions? options,
    FHttpCancelToken? cancelToken,
    dio.ProgressCallback? onReceiveProgress,
  });

  /// POST 请求
  ///
  /// [path] 请求地址
  ///
  /// [data] POST 数据
  ///
  /// [queryParameters] 请求参数
  ///
  /// [options] 单次配置，仅本次请求有效
  ///
  /// [cancelToken] 取消标识
  Future<FHttpResponse<T>> post<T>(
    String path, {
    data,
    Map<String, dynamic>? queryParameters,
    FHttpScopedOptions? options,
    FHttpCancelToken? cancelToken,
    dio.ProgressCallback? onSendProgress,
    dio.ProgressCallback? onReceiveProgress,
  });

  /// Put 请求
  ///
  /// [path] 请求地址
  ///
  /// [data] POST 数据
  ///
  /// [queryParameters] 请求参数
  ///
  /// [options] 单次配置，仅本次请求有效
  ///
  /// [cancelToken] 取消标识
  Future<FHttpResponse<T>> put<T>(
    String path, {
    data,
    Map<String, dynamic>? queryParameters,
    FHttpScopedOptions? options,
    FHttpCancelToken? cancelToken,
    dio.ProgressCallback? onSendProgress,
    dio.ProgressCallback? onReceiveProgress,
  });

  /// Delete 请求
  ///
  /// [path] 请求地址
  ///
  /// [queryParameters] 请求参数
  ///
  /// [options] 单次配置，仅本次请求有效
  ///
  /// [cancelToken] 取消标识
  Future<FHttpResponse<T>> delete<T>(
    String path, {
    Map<String, dynamic>? queryParameters,
    FHttpScopedOptions? options,
    FHttpCancelToken? cancelToken,
  });
}

/// FHttp Global instance
final FHttp fhttp = FHttp(FHttpBaseOptions(timeout: 3 * 1000));

class FHttpCancelToken extends dio.CancelToken {}
