import 'package:universal_html/js.dart';

import 'package:fis_common/web/js_utils.dart';

const _OBJ_KEY = "FisShellApi";

/// 壳子Bridge api帮助类
class ShellApiHelper {
  static ShellApiHelper? _singleton;

  static ShellApiHelper get _ins {
    _singleton ??= ShellApiHelper._();
    return _singleton!;
  }

  late final bool _hasShellObj;

  ShellApiHelper._() {
    _hasShellObj = context.hasProperty(_OBJ_KEY);
  }

  JsObject? get _shellApi {
    if (_hasShellObj) {
      return context[_OBJ_KEY];
    } else {
      return null;
    }
  }

  /// 请求壳子 Api
  ///
  /// [method] 方法名
  ///
  /// [args] 可选参数，单变量/变量数组
  ///
  /// *!!! 实体类参数，需实现`toJson`，且`toJson`返回类型是`Map<String, dynamic>`
  static Future call(String method, [dynamic args]) => _ins._call(method, args);

  Future _call(String method, [dynamic args]) async {
    if (_shellApi == null) {
      throw Exception("Current environment is not FisShell program.");
    }
    final argArr = _convertArgs(args);
    return await _shellApi!.invokeJsPromise(method, argArr);
  }

  List _convertArgs(dynamic args) {
    if (args == null) {
      return [];
    }
    if (args is List) {
      return _convertListArg(args);
    }
    return [_convertArgItem(args)];
  }

  List _convertListArg(List arr) {
    final rst = [];
    for (var item in arr) {
      rst.add(_convertArgItem(item));
    }
    return rst;
  }

  dynamic _convertArgItem(dynamic val) {
    if (val is num || val is String || val is bool) {
      return val;
    }
    if (val is Enum) {
      return val.index;
    }
    if (val is List || val is Set) {
      return _convertListArg(val);
    }
    if (val is Map) {
      return _convertMapArg(val);
    }
    if (val is Function) {
      return JsFunction.withThis(val);
    }
    return _convertObjectArg(val);
  }

  JsObject _convertMapArg(Map map) {
    return JsObject.jsify(map);
  }

  JsObject _convertObjectArg(dynamic obj) {
    Map map;
    try {
      map = obj.toJson();
    } catch (e) {
      map = {};
    }
    return _convertMapArg(map);
  }
}
