import 'dart:io';

import 'package:intl/intl.dart';
import 'package:path_provider/path_provider.dart';

import 'logger.dart';

class WebLogger extends Logger {
  @override
  Future<void> write(LogLevel level, String msg) async {}
}

class WebShellLogger extends Logger {
  @override
  Future<void> write(LogLevel level, String msg) async {}
}

class NativeLogger extends Logger {
  IOSink? _ioSink;
  late final String _filePath;

  @override
  Future<void> write(LogLevel level, String msg) async {
    _keepIOSink();

    final content = buildLogContent(level, msg);
    _ioSink!.writeln(content);
    // await _ioSink!.flush();
    // _ioSink = null;
  }

  @override
  init() async {
    await super.init();
    print('Logger_IO init.');
    Directory? dir;
    if (Platform.isAndroid) {
      dir = await getExternalStorageDirectory();
    }
    if (dir == null) {
      dir = await getApplicationDocumentsDirectory();
    }
    final now = DateTime.now();
    final logTag = DateFormat('yyyyMMddHHmmss_SSS').format(now);
    _filePath = '${dir.path}/log_$logTag.log';
    // final file = File(_filePath);
    // _ioSink = file.openWrite(mode: FileMode.append);
  }

  _keepIOSink() {
    if (_ioSink == null) {
      final file = File(_filePath);
      _ioSink = file.openWrite(mode: FileMode.append);
    }
  }

  @override
  dispose() async {
    await _ioSink?.close();
    await super.dispose();
  }
}
