// ignore: avoid_web_libraries_in_flutter
import 'dart:js' as js;

import 'logger.dart';

class WebLogger extends Logger {
  static Map<LogLevel, String> _logLevelColors = {
    LogLevel.Verbose: "#333300",
    LogLevel.Degbug: "#0099cc",
    LogLevel.Info: "#00cc33",
    LogLevel.Warn: "#cc9900",
    LogLevel.Error: "#cc0000",
    LogLevel.UserOperate: "#993399",
  };

  WebLogFunc? _logFunc;

  @override
  Future<void> write(LogLevel level, String msg) async {
    var color = _logLevelColors[level];
    _logFunc?.call([
      buildLogContent(level, msg),
      'font-size:14px;color:$color;',
      'font-size:12px;color:#333333;',
    ]);
  }

  @override
  String buildLogContent(LogLevel level, String msg) {
    var levelTag = getLevelTag(level);
    var timeStr = getLogTimeString();
    return "%c[$timeStr]-$levelTag; %c$msg";
  }

  @override
  init() async {
    await super.init();
    print('Logger_Web init!');
    if (js.context.hasProperty('console')) {
      final console = js.context['console'] as js.JsObject;
      if (console.hasProperty('log')) {
        _logFunc = (args) => console.callMethod('log', args);
      }
    }
  }
}

typedef void WebLogFunc(List<dynamic> args);

typedef Future<void> WebLogFuncAsync(List<dynamic> args);

class WebShellLogger extends Logger {
  WebLogFuncAsync? _logFunc;

  @override
  Future<void> write(LogLevel level, String msg) async {
    final content = buildLogContent(level, msg);
    await _logFunc?.call([content]);
  }

  @override
  init() async {
    await super.init();
    print('Logger_WebShell init!');
    if (js.context.hasProperty('FisShellApi')) {
      final api = js.context['FisShellApi'] as js.JsObject;
      if (api.hasProperty('writeLog'))
        _logFunc = (args) async => await api.callMethod('writeLog', args);
    }
  }
}

class NativeLogger extends Logger {
  @override
  Future<void> write(LogLevel level, String msg) async {}
}
